<?php
/**
 * 2018-2022 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2022 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */


/**
 * NOTICE:
 * This is an explicit modification of the native Prestashop module: ps_checkpayment
 * its natural position should be: paymentOptions, displayPaymentReturn
 */


  /**
  * for DEBUG: if you use mxb_tools module, you'll find the log file in folder: /module/mxb_tools/log/
  *         require_once _PS_MODULE_DIR_.'mxb_tools/classes/MxbToolsDebug.php';
  * and then use this function:
  *        MxbToolsDebug::WriteLog($classname, $function, $message, $bypass_settings = false)
  *    i.e:
  *        MxbToolsDebug::WriteLog($this->name, 'className::__construct()', 'START', true);
  */

 /**
 * DUPLICATE & CUSTOMIZE:
 * create a new copy of this module to have more menus in your Prestashop
 * there are a few steps: follow it down in the first part of the code (class declarations and __construct() function)
 *
 * module name root MUST remains mxb_delaypay_
 * just append to it a string (lowercase, only letter): this string will also must be set in $this->customName variabile and in other part of the code (follow next instructions)
 *
 * so, for $this->customName = "30days",
 * the modulename will be "mxb_delaypay_30days"
 *
 * for the following instructions, "yourname" custom name will be used to make examples
 */

/* DUPLICATE & CUSTOMIZE 1/3: first, remember to change main module file Mxb_Delaypay_bankwire.php
                                  the name of the file as: mxb_delaypay_[CustomName].php
                                  its class declaration: change the class name as the module name: Mxb_Delaypay_[yourname]
                                  main variabiles in __construct(): set the class root name as your "yourname": $this->customName = 'yourname', and vars name, displayName
                              then:
                                  config.xml -> the module name
                                  payment.php -> set the right module name (Mxb_Delaypay_[Yourname]) (it's deprecated but it needs to be correctly compiled)
                                  controllers\front\payment.php -> Mxb_Delaypay_[Yourname]PaymentModuleFrontController
                                  controllers\front\validation.php -> Mxb_Delaypay_[Yourname]ValidationModuleFrontController
                                  mails\it\mxb_delaypay_template.html (and folder mails\en / file .html + .txt) -> edit files and customize email template
*/

// require_once _PS_MODULE_DIR_.'mxb_tools/classes/MxbToolsDebug.php';
use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_')) {
    exit;
}

 /**
  * ###### DUPLICATE & CUSTOMIZE 2/3: class declaration: change the class name as the module name: Mxb_Delaypay_[CustomName]
  */
class Mxb_Delaypay_Riba extends PaymentModule
{
    const MODULE_NAME_ROOT = 'mxb_delaypay';
    public $documentationUrl = 'https://www.mxbinformatica.it/prestashop-mxb-delaypay/';
    public $thumbnailUrl = 'https://www.mxbinformatica.it/ps_modules/thumbs/mxb_delaypay.jpg';

    // the following version variable is for development, used do update databases and other stuff without reinstalling the module
    // please check checkForUpdates() function
    // use int value !
    const MXB_DEV_VERSION = 1;
    
    private $_html = '';
    private $_postErrors = array();

   // public $checkName;
    //public $address;
    public $ribaName;
    public $ribaInstructions;
    public $ribaDays;
    public $ribaDaysCustomer;
    public $ribaOrderStatus;
    public $customerList;
    public $extra_mail_vars;
    public $tableName = null;
    public $dev_version = null;

    public function __construct()
    {
        /* DUPLICATE & CUSTOMIZE 3/3: main variabiles: set the class root name as "yourname"  */
        $this->customName = 'riba';
        // PS module verification doesn't allow to construct name with other variables
        $this->name = 'mxb_delaypay_riba';
        // PS module verification need a displayName identical to the one into config.xml
        $this->displayName = 'MXB Delayed Payment: riba'; 

        /* DUPLICATE & CUSTOMIZE: no more changes are needed from here */
        $this->rootName = 'mxb_delaypay_';
        // PS module verification doesn't allow to construct name with other variables
        //$this->name = $this->rootName . $this->customName;
        $this->tableName = $this->rootName . $this->customName;
        $this->orderStateName = $this->l('Delayed payment with') . ' ' . str_replace("_", " ", $this->customName);

        $this->version = '1.0.0';
        $this->dev_version = self::MXB_DEV_VERSION;
        
        $this->author = 'MXB Informatica';
        $this->tab = 'payments_gateways';
        $this->is_eu_compatible = 1;

        $this->controllers = array('payment', 'validation');

        $this->currencies = true;
        $this->currencies_mode = 'checkbox';
        
        $this->bootstrap = true;
        parent::__construct();

        // PS module verification need a displayName identicat to the one into config.xml but we can change it here
        $this->displayName = $this->l('MXB delayed payment') . ': ' . str_replace("_", " ", $this->customName);
        $this->description = $this->l('This module allows you to accept delayed payments');
        $this->confirmUninstall = $this->trans('Are you sure you want to delete these details?', array(), 'Modules.Checkpayment.Admin');
        $this->ps_versions_compliancy = array('min' => '1.7.1.0', 'max' => _PS_VERSION_);

        $this->CONFIG_VERSION = $this->name . '_VERSION';
        $this->CONFIG_RIBA_NAME = $this->name . '_PAYMENT_NAME';
        $this->CONFIG_RIBA_INSTRUCTIONS = $this->name . '_PAYMENT_INSTRUCTIONS';
        $this->CONFIG_RIBA_DAYS = $this->name . '_PAYMENT_RIBA_DAYS';
        $this->CONFIG_RIBA_ORDER_STATUS = $this->name . '_ORDER_STATUS';
        // $this->CONFIG_RIBA_CUSTOMER_LIST = $this->name . '_CUSTOMER_LIST'; // obsoleto
        $this->CONFIG_RIBA_CUSTOMER_LIST_ARR = $this->name . '_CUSTOMER_LIST_ARR';
        /*
        $config = Configuration::getMultiple(array(
            $this->CONFIG_RIBA_NAME,
            $this->CONFIG_RIBA_INSTRUCTIONS,
            $this->CONFIG_RIBA_DAYS));
        */

        $this->checkForUpdates();

        $config = $this->getConfigFieldsValues();
        $id_lang = (int)Context::getContext()->language->id;

        if (isset($config[$this->CONFIG_RIBA_NAME][$id_lang])) {
            $this->ribaName = $config[$this->CONFIG_RIBA_NAME][$id_lang];
        }
        if (isset($config[$this->CONFIG_RIBA_INSTRUCTIONS][$id_lang])) {
            $this->ribaInstructions = $config[$this->CONFIG_RIBA_INSTRUCTIONS][$id_lang];
        }
        if (isset($config[$this->CONFIG_RIBA_DAYS][$id_lang])) {
            $this->ribaDays = $config[$this->CONFIG_RIBA_DAYS][$id_lang];
        }
        //if (isset($config[$this->CONFIG_RIBA_DAYS])) {
        //    $this->ribaDays = (int)$config[$this->CONFIG_RIBA_DAYS];
        //}
        if (isset($config[$this->CONFIG_RIBA_ORDER_STATUS])) {
            $this->ribaOrderStatus = (int)$config[$this->CONFIG_RIBA_ORDER_STATUS];
        } else {
            $this->ribaOrderStatus = 0;
        }

        if (isset($config[$this->CONFIG_RIBA_CUSTOMER_LIST_ARR])) {
            $this->customerList = $config[$this->CONFIG_RIBA_CUSTOMER_LIST_ARR];
            $this->ribaDaysCustomer = $this->checkCustomer(null, $config[$this->CONFIG_RIBA_CUSTOMER_LIST_ARR]);
            // $this->ribaDaysCustomer = $this->checkCustomer();
            if ($this->ribaDaysCustomer === true || $this->ribaDaysCustomer === false) {
                $this->ribaDaysCustomer = $this->ribaDays;
            }
        } else {
           $this->ribaDaysCustomer = $this->ribaDays;
        }

        if ((string)(int)$this->ribaDaysCustomer === (string)$this->ribaDaysCustomer) {
            $this->ribaDaysCustomer .= ' ' . $this->l('days');
        }
        // MxbToolsDebug::WriteLog($this->name, '__construct', 'ribaDaysCustomer: ' . $this->ribaDaysCustomer . ' CUS ARR: ' . print_r($config[$this->CONFIG_RIBA_CUSTOMER_LIST_ARR], true), true);

        if ((!isset($this->ribaName) || !isset($this->ribaInstructions) || !isset($this->ribaDays)
            || empty($this->ribaName) || empty($this->ribaInstructions) || empty($this->ribaDays))) {
            $this->warning = $this->l('Please compile all the required fields before using this module.');
        }
        if (!count(Currency::checkPaymentCurrencies($this->id))) {
            $this->warning = $this->l('No currency has been set for this module.');
        }

        // unusefull ? (check controller/front/validation.php)
        $this->extra_mail_vars = array(
            /*
            '{check_name}' => Configuration::get('CHEQUE_NAME'),
            '{check_address}' => Configuration::get('CHEQUE_ADDRESS'),
            '{check_address_html}' => Tools::nl2br(Configuration::get('CHEQUE_ADDRESS')),
            */
            '{payment_name}' => $this->ribaName,
            '{payment_instructions}' => $this->ribaInstructions,
            '{payment_days}' => $this->ribaDaysCustomer,
        );
        
        
        // REMOVE IN PRODUCTION MODE: set module files JS, TPL, CSS
        //$this->installModuleFiles();

    }

    public function install()
    {
        $ret = parent::install()
            && $this->registerHook('paymentOptions')
            && $this->registerHook('paymentReturn')
        ;
        if ($ret) {
            //Configuration::deleteByName($this->CONFIG_RIBA_ORDER_STATUS);
            $this->installModuleFiles();
            $this->checkMailTemplate();
            $this->setDefaultValues();
            /* Creates tables */
            $this->createTables();
        }
        return $ret;
    }

    public function uninstall()
    {
        /* Deletes Module */
        if (parent::uninstall()) {
            /* Deletes tables */
            $res = $this->deleteTables();

            /* Unsets configuration */
            $res &= Configuration::deleteByName($this->CONFIG_RIBA_NAME)
                && Configuration::deleteByName($this->CONFIG_RIBA_INSTRUCTIONS)
                && Configuration::deleteByName($this->CONFIG_RIBA_DAYS)
                && Configuration::deleteByName($this->CONFIG_RIBA_ORDER_STATUS);
                // && Configuration::deleteByName($this->CONFIG_RIBA_CUSTOMER_LIST); // obsoleto
            return (bool)$res;
        }

        return false;
    }

    /**
     * Creates tables
     */
    protected function createTables()
    {
        /* Slides */
        $res = (bool)Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.$this->tableName . '`(
                `id_' . $this->tableName . '` int(10) unsigned NOT NULL AUTO_INCREMENT,
                `id_customer` int(10) unsigned NOT NULL,
                `days` varchar(100),
                PRIMARY KEY (`id_' . $this->tableName . '`, `id_customer`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');

        return $res;
    }

    /**
     * deletes tables
     */
    protected function deleteTables()
    {
        return Db::getInstance()->execute('
            DROP TABLE IF EXISTS `'._DB_PREFIX_.$this->tableName . '`;
        ');
    }

    private function setDefaultValues()
    {
        $config = $this->getConfigFieldsValues();
        foreach ($config[$this->CONFIG_RIBA_NAME] as &$val) {
            if ($val === false) {
                $val = $this->customName;
            }
        }
        foreach ($config[$this->CONFIG_RIBA_DAYS] as &$val) {
            if ($val === false) {
                $val = 30;
            }
        }
        foreach ($config[$this->CONFIG_RIBA_INSTRUCTIONS] as &$val) {
            if ($val === false) {
                $val = 'as agreed';
            }
        }
        Configuration::updateValue($this->CONFIG_RIBA_DAYS, $config[$this->CONFIG_RIBA_DAYS]);
        Configuration::updateValue($this->CONFIG_RIBA_NAME, $config[$this->CONFIG_RIBA_NAME]);
        Configuration::updateValue($this->CONFIG_RIBA_INSTRUCTIONS, $config[$this->CONFIG_RIBA_INSTRUCTIONS]);
    }

    private function checkMailTemplate()
    {
        // create order status
        $orderStatusId = Configuration::get($this->CONFIG_RIBA_ORDER_STATUS);
        if ($this->checkOrderState($orderStatusId)) {
            return true;
        }

        $orderStatusId = $this->addOrderState(Configuration::get($this->CONFIG_RIBA_ORDER_STATUS));
        if ($orderStatusId === false) {
            return;
        }
        // MxbToolsDebug::WriteLog('mxb_payment_riba_30d', 'checkMailTemplate', 'orderStatusId: ' . $orderStatusId, true);

        // update order status id
        Configuration::updateValue($this->CONFIG_RIBA_ORDER_STATUS, $orderStatusId);
        $this->ribaOrderStatus = $orderStatusId;

        //if ((int)$orderStatusId > 0) {
        //    return true;
        //}

        // copy mail templates into theme
        $this->copyMailTemplate('en');
        $this->copyMailTemplate('it');

        return true;
    }


    private function copyMailTemplate($langFolder = 'en')
    {
        // copy mail templates into theme
        $sourceFolder = $this->name.DIRECTORY_SEPARATOR.'mails'.DIRECTORY_SEPARATOR.$langFolder;
        $sourceFileName = $this->rootName . 'template'; // add extension dinamically during copy command (html / txt)
        $targetFolder = 'mails'.DIRECTORY_SEPARATOR.$langFolder;
        $targetFileName = $this->name; // add extension dinamically during copy command (html / txt)

        //$source = $sourceFolder.DIRECTORY_SEPARATOR.$sourceFileName;
        //$target = $targetFolder.DIRECTORY_SEPARATOR.$targetFileName;
        // MxbToolsDebug::WriteLog($this->name, 'copyMailTemplate', 'file to copy FROM: ' . _PS_THEME_DIR_.$source.'.html' . ', TO: ' . _PS_THEME_DIR_.$target.'.html', true);

        // set current template
        $this->copyMailTemplateFiles(_PS_THEME_DIR_, $sourceFolder, $sourceFileName, $targetFolder, $targetFileName);
        $classicThemeDir = _PS_THEME_DIR_.'../classic/';
        if (!file_exists($classicThemeDir.$targetFolder.DIRECTORY_SEPARATOR.$targetFileName.'.html')) {
            $this->copyMailTemplateFiles($classicThemeDir, $sourceFolder, $sourceFileName, $targetFolder, $targetFileName);
        }
        /*
        if (!file_exists(_PS_THEME_DIR_.$targetFolder)) {
            mkdir(_PS_THEME_DIR_.$targetFolder, 0777, true);
        }
        if (!file_exists(_PS_THEME_DIR_.$target.'.html')) {
            copy(_PS_MODULE_DIR_.$source.'.html', _PS_THEME_DIR_.$target.'.html');
            copy(_PS_MODULE_DIR_.$source.'.txt', _PS_THEME_DIR_.$target.'.txt');
            return true;
        }
        //set also classic (base) template
        $classicDir = _PS_THEME_DIR_.'../classic/';
        if (!file_exists($classicDir.$target.'.html')) {
            if (!file_exists($classicDir.$targetFolder)) {
                mkdir($classicDir.$targetFolder, 0777, true);
            }
            if (!file_exists($classicDir.$target.'.html')) {
                copy(_PS_MODULE_DIR_.$source.'.html', $classicDir.$target.'.html');
                copy(_PS_MODULE_DIR_.$source.'.txt', $classicDir.$target.'.txt');
                return true;
            }
        }
        */
        return true;
    }

    private function copyMailTemplateFiles($themeDir, $sourceFolder, $sourceFileName, $targetFolder, $targetFileName)
    {
        $source = $sourceFolder.DIRECTORY_SEPARATOR.$sourceFileName;
        $target = $targetFolder.DIRECTORY_SEPARATOR.$targetFileName;
        
        if (!file_exists($themeDir.$targetFolder)) {
            mkdir($themeDir.$targetFolder, 0777, true);
        }
        if (!file_exists($themeDir.$target.'.html')) {
            copy(_PS_MODULE_DIR_.$source.'.html', $themeDir.$target.'.html');
            copy(_PS_MODULE_DIR_.$source.'.txt', $themeDir.$target.'.txt');
            return true;
        }
        return false;
    }

    private function checkOrderState($id = 0)
    {
        if ((int)$id === 0) {
            return false;
        }
        $states = OrderState::getOrderStates((int)$this->context->language->id);

        // check if order state exist
        foreach ($states as $state) {
            //if (in_array($name, $state)) {
            if ((int)$id === (int)$state['id_order_state']) {
                return true;
                //$state_exist = true;
                //break;
            }
        }
        return false;
    }

    private function addOrderState($id = 0)
    {
        // state already exists ? If the state does not exist, we create it.
        if ($this->checkOrderState($id)) {
            return false;
        }

        // create new order state
        $order_state = new OrderState();
        $order_state->color = '#32CD32';
        $order_state->send_email = true;
        $order_state->module_name = $this->name; //'name if your module';
        $order_state->template = $this->name; //'name of your email template';
        $order_state->name = array();
        $languages = Language::getLanguages(false);
        foreach ($languages as $language) {
            $order_state->name[$language['id_lang']] = $this->orderStateName;
        }

        // Update object
        $order_state->add();

        return $order_state->id;
    }

    public function installModuleFiles()
    {
        $this->installModuleFile('translations/it.php');
        $this->installModuleFile('views/templates/hook/infos.tpl');
        $this->installModuleFile('views/templates/hook/payment.tpl');
        $this->installModuleFile('views/templates/hook/payment_return.tpl');
        $this->installModuleFile('views/templates/hook/customer_list.tpl');
        $this->installModuleFile('views/templates/front/payment_infos.tpl');
        return true;
    }



    public function installModuleFile($module_file = 'translations/it.php', $overwrite = true)
    {
        $filedir = _PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR;
        $filename_src = $filedir . $module_file . '_'; // add underscore to open the source unmodifyed file
        $filename_tgt = $filedir . $module_file;
        $searchString = '_' . $this->rootName;

        if (!file_exists($filename_src)) {
            return false;
        }
        if (file_exists($filename_tgt) && !$overwrite) {
            return false;
        }

        $myfile = fopen($filename_src, "r");
        $filesize = filesize($filename_src);
        if ((int)$filesize === 0) {
            return false;
        }
        $content = fread($myfile, $filesize);
        fclose($myfile);

        // change default class with custom class and save file
        // text to replace not found ?
        //if (strpos($content, $searchString) === false)
        //    return true;
        // already replaced (replace text is already present) ?
        if (strpos($content, $this->name) === false) {
            $content = str_replace($searchString, $this->name, $content);
        }

        // update the real file
        $myfile = fopen($filename_tgt, "w");
        fwrite($myfile, $content);
        fclose($myfile);

        return true;
    }

    private function _displayCheck()
    {
        return $this->display(__FILE__, './views/templates/hook/infos.tpl');
    }

    public function getContent()
    {
        // DEBUG: INSTALL ONCE AND REMOVE
        //$this->setDefaultValues();
        //Configuration::deleteByName($this->CONFIG_RIBA_ORDER_STATUS);
        //$this->checkMailTemplate();
        //$this->installModuleFiles();

        $this->_html = '';

        /* Validate & process */
        if (Tools::isSubmit('btnSubmit') || Tools::isSubmit('delete_id_delaypay_customer') ||
            Tools::isSubmit('submitCustomer')
        ) {
            
            if ($this->_postValidation()) {
                if (!count($this->_postErrors)) {
                    $this->_postProcess();
                } else {
                    foreach ($this->_postErrors as $err) {
                        $this->_html .= $this->displayError($err);
                    }
                }
                $this->smarty->assign([
                    'mxb_payment_riba_module_name' => $this->name,
                    'order_state_name' => $this->orderStateName,
                    //'hookName' => $hookName,
                ]);
                $this->_html .= $this->_displayCheck();
                $this->_html .= $this->renderForm();
                $this->_html .= $this->renderList();
            } else {
                $this->_html .= $this->renderAddForm();
            }
            $this->clearCache();
        } elseif (Tools::isSubmit('addCustomer') || (Tools::isSubmit('id_delaypay_customer') && $this->getCustomer((int)Tools::getValue('id_delaypay_customer')))) {
            $this->_html .= $this->renderAddForm();
        } else {
            $this->smarty->assign([
                'mxb_payment_riba_module_name' => $this->name,
                'order_state_name' => $this->orderStateName,
                //'hookName' => $hookName,
            ]);
            $this->_html .= $this->_displayCheck();
            $this->_html .= $this->renderForm();
            $this->_html .= $this->renderList();
        }

        return $this->_html;
    }

    private function _postValidation()
    {
        $errors = array();
        if (Tools::isSubmit('btnSubmit')) {
            $languages = Language::getLanguages(false);
            $err = false;

            if (!Tools::getValue($this->CONFIG_RIBA_ORDER_STATUS) || (int)Tools::getValue($this->CONFIG_RIBA_ORDER_STATUS) === 0) {
                $errors[] = $this->l('The "Order status" field is required.');
            } else {
                foreach ($languages as $lang) {
                    if (!Tools::getValue($this->CONFIG_RIBA_NAME . '_'.$lang['id_lang'])) {
                        $errors[] = $this->l('The "Payment name" field is required for all languages.');
                        break;
                    } elseif (!Tools::getValue($this->CONFIG_RIBA_INSTRUCTIONS . '_'.$lang['id_lang'])) {
                        $errors[] = $this->l('The "Payment instructions" field is required for all languages.');
                        break;
                    } elseif (!Tools::getValue($this->CONFIG_RIBA_DAYS . '_'.$lang['id_lang'])) {
                        $errors[] = $this->l('The "Days of delay" field is required for all languages.');
                        break;
                    }
                }
            }
        
        } elseif (Tools::isSubmit('submitCustomer')) {
            /* Checks state (active_customer) 
            if (!Validate::isInt(Tools::getValue('active_customer')) || (Tools::getValue('active_customer') != 0 && Tools::getValue('active_customer') != 1)) {
                $errors[] = $this->l('Invalid block state.');
            }*/
            /* Checks state (id_customer) */
            if (!Validate::isInt(Tools::getValue('id_customer'))) {
                $errors[] = $this->l('Invalid block id_customer.');
            }

            /* If edit : checks id_delaypay_customer */
            if (Tools::isSubmit('id_delaypay_customer')) {
                if (!Validate::isInt(Tools::getValue('id_delaypay_customer')) && !$this->getCustomer(Tools::getValue('id_delaypay_customer'))) {
                    $errors[] = $this->l('Invalid block ID');
                }
            }

        } elseif (Tools::isSubmit('delete_id_slide') && (!Validate::isInt(Tools::getValue('delete_id_slide')) || !$this->getCustomer((int)Tools::getValue('delete_id_slide')))) {
            $errors[] = $this->l('Invalid block ID');
        }
        
        /* Display errors if needed */
        if (count($errors)) {
            $this->_html .= $this->displayError(implode('<br />', $errors));

            return false;
        }

        /* Returns if validation is ok */

        return true;
    }

    private function _postProcess()
    {
        $errors = array();

        if (Tools::isSubmit('btnSubmit')) {
            $languages = Language::getLanguages(false);
            $values = array();

            foreach ($languages as $lang) {
                $values[$this->CONFIG_RIBA_NAME][$lang['id_lang']] = Tools::getValue($this->CONFIG_RIBA_NAME . '_'.$lang['id_lang']);
                $values[$this->CONFIG_RIBA_INSTRUCTIONS][$lang['id_lang']] = Tools::getValue($this->CONFIG_RIBA_INSTRUCTIONS . '_'.$lang['id_lang']);
                $values[$this->CONFIG_RIBA_DAYS][$lang['id_lang']] = Tools::getValue($this->CONFIG_RIBA_DAYS . '_'.$lang['id_lang']);
            }
            Configuration::updateValue($this->CONFIG_RIBA_DAYS, $values[$this->CONFIG_RIBA_DAYS]);
            Configuration::updateValue($this->CONFIG_RIBA_NAME, $values[$this->CONFIG_RIBA_NAME]);
            Configuration::updateValue($this->CONFIG_RIBA_INSTRUCTIONS, $values[$this->CONFIG_RIBA_INSTRUCTIONS]);
            Configuration::updateValue($this->CONFIG_RIBA_ORDER_STATUS, Tools::getValue($this->CONFIG_RIBA_ORDER_STATUS));
            // Configuration::updateValue($this->CONFIG_RIBA_CUSTOMER_LIST, Tools::getValue($this->CONFIG_RIBA_CUSTOMER_LIST)); // obsoleto
            
            // $this->clearCache();
            $this->_html .= $this->displayConfirmation($this->trans('Settings updated', array(), 'Admin.Notifications.Success'));

        } elseif (Tools::isSubmit('submitCustomer')) {
            /* Sets ID if needed */
            if (Tools::getValue('id_delaypay_customer')) {
                $customer = $this->getCustomer((int)Tools::getValue('id_delaypay_customer'));
                if (!$customer) {
                    $this->_html .= $this->displayError($this->l('Invalid block ID'));
                    return false;
                }
            } else {
                $customer = $this->newCustomer();
            }
            /* Sets id_customer */
            $customer['id_customer'] = (int)Tools::getValue('id_customer');
            
            if ($this->customerExists($customer['id_customer'], $customer['id_' . $this->tableName])) {
                $this->_html .= $this->displayError($this->l('Selected customer is already configured'));
                return false;
            }
            
            
            /* Sets days */
            $customer['days'] = (string)Tools::getValue('days');

            /* Processes if no errors  */
            if (!$errors) {
                /* Adds */
                if (!Tools::getValue('id_delaypay_customer')) {
                    if (!$this->addCustomer($customer)) {
                        $errors[] = $this->displayError($this->l('The block could not be added.'));
                    }
                } elseif (!$this->updateCustomer($customer)) {
                    $errors[] = $this->displayError($this->l('The block could not be updated.'));
                }
                $this->clearCache();
            }
        } elseif (Tools::isSubmit('delete_id_delaypay_customer')) {
            $customer = $this->getCustomer((int)Tools::getValue('delete_id_delaypay_customer'));
            $res = $this->deleteCustomer((int)$customer['id_' . $this->tableName]);
            $this->clearCache();
            if (!$res) {
                $this->_html .= $this->displayError('Could not delete.');
            } else {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=1&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
            }
        }

        /* Display errors if needed */
        if (count($errors)) {
            $this->_html .= $this->displayError(implode('<br />', $errors));
        } elseif (Tools::isSubmit('submitCustomer') && Tools::getValue('id_delaypay_customer')) {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=4&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        } elseif (Tools::isSubmit('submitCustomer')) {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=3&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        }
    }

    public function clearCache()
    {
        // $this->_clearCache($this->templateFile);
    }

    public function getCustomers($days = null)
    {
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
            SELECT *
            FROM `'._DB_PREFIX_.$this->tableName . '`'
        );
        if (is_array($result) && count($result)) {
            return $this->completeCustomerArray($result, $days);
        }

        return null;
    }

    public function getCustomersFull($days = null)
    {
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
            SELECT dc.*, c.email as email, c.firstname as firstname, c.lastname as lastname
            FROM `'._DB_PREFIX_ . $this->tableName . '` dc
            LEFT JOIN `'._DB_PREFIX_ . 'customer` c ON (dc.id_customer = c.id_customer)
        ');
        if (is_array($result) && count($result)) {
            return $this->completeCustomerArray($result, $days);
        }

        return null;
    }

    public function getCustomer($id_delaypay_customer = null, $days = null)
    {
        if ((int)$id_delaypay_customer <= 0) {
            return false;
        }
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
            SELECT *
            FROM `'._DB_PREFIX_.$this->tableName . '`
            WHERE `id_' . $this->tableName . '` = ' . (int)$id_delaypay_customer
        );
        if (is_array($result) && count($result)) {
            return $this->completeCustomerArray([$result[0]], $days)[0];
        }

        return null;
    }


    public function completeCustomerArray($array, $days = null)
    {
        if (!$days) {
            $days = '';
        }

        foreach ($array as &$res) {
            $res['id'] = $res['id_' . $this->tableName];
            if (isset($res['email'])) {
                $res['customer_description'] = $res['email'] . ' - ' . $res['firstname'] . ' ' . $res['lastname'];
            }
            /*
            if (strlen((string)$res['days'])) {
                $daysStr = $res['days'];
            } else {
                $daysStr = $days;
            }
            $res['days_description'] = ((string)(int)$daysStr === (string)$daysStr 
                ? $daysStr . ' ' . $this->l('days')
                : $daysStr);
            */
            $res['days_description'] = $this->getDaysString($res['days'], $days);
        }

        return $array;
    }

    public function getDaysString($customer_days, $days = null)
    {
        if (!$days) {
            $days = '';
        }
        if (strlen((string)$customer_days)) {
            $daysStr = $customer_days;
        } else {
            $daysStr = $days;
        }
        return ((string)(int)$daysStr === (string)$daysStr 
            ? $daysStr . ' ' . $this->l('days')
            : $daysStr);
    }


    public function customerExists($id_customer = null, $id_delaypay_customer = null)
    {
        if ((int)$id_customer <= 0) {
            return false;
        }
        $addSql = ((int)$id_delaypay_customer > 0 ? ' AND id_' . $this->tableName . ' <> ' . $id_delaypay_customer . ' ' : '');
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
            SELECT *
            FROM `'._DB_PREFIX_.$this->tableName . '`
            WHERE `id_customer` = ' . (int)$id_customer
            . $addSql
        );

        return (is_array($result) && count($result));
    }

    public function newCustomer() {
        return [
            'id_' . $this->tableName => 0,
            'id_customer' => 0,
            'days' => '',
        ];
    }

    public function updateCustomer($customer = null)
    {
        if (!$customer) {
            return false;
        }
        $id_delaypay_customer = (int)$customer['id_' . $this->tableName];
        if ($id_delaypay_customer === 0 || (int)$customer['id_customer'] === 0) {
            return false;
        }
        $cus = $this->getCustomer($id_delaypay_customer);
        if (!$cus) {
            return false;
        }
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->execute('
            UPDATE `'._DB_PREFIX_.$this->tableName . '`
            SET id_customer=' . (int)$customer['id_customer'] . ',
            days="' . (string)$customer['days'] . '"
            WHERE `id_' . $this->tableName . '` = ' . (int)$id_delaypay_customer
        );

        return (bool)$result;
    }

    public function addCustomer($customer = null)
    {
        if (!$customer) {
            return false;
        }
        if ((int)$customer['id_customer'] === 0) {
            return false;
        }
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->execute('
            INSERT INTO `'._DB_PREFIX_.$this->tableName . '`
            (id_customer, days)
            VALUES (
            ' . (int)$customer['id_customer'] . ',
            "' . (string)$customer['days'] . '")'
        );

        return (bool)$result;
    }

    public function deleteCustomer($id_delaypay_customer = null)
    {
        if ((int)$id_delaypay_customer <= 0) {
            return false;
        }
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->execute('
            DELETE FROM `'._DB_PREFIX_.$this->tableName . '`
            WHERE `id_' . $this->tableName . '` = ' . (int)$id_delaypay_customer
        );

        return $result;
    }

    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }
        if (!$this->checkCurrency($params['cart'])) {
            return;
        }
        // MxbToolsDebug::WriteLog($this->name, 'hookPaymentOptions', 'checkCustomer: ' . $this->checkCustomer(), true);

        if (!$this->checkCustomer()) {
            return;
        }
        $this->smarty->assign(
            $this->getTemplateVars()
        );
        $newOption = new PaymentOption();
        $newOption->setModuleName($this->name)
            ->setCallToActionText($this->l('Pay with') . ' ' . $this->getPaymentFullName()) //Configuration::get($this->CONFIG_RIBA_NAME)
            ->setAction($this->context->link->getModuleLink($this->name, 'validation', array(), true))
            ->setAdditionalInformation($this->fetch('module:' . $this->name . '/views/templates/front/payment_infos.tpl'));
        // MxbToolsDebug::WriteLog($this->name, 'hookPaymentOptions', 'END - checkCustomer: ' . $this->checkCustomer(), true);

        return [$newOption];
    }

    public function hookPaymentReturn($params)
    {
        if (!$this->active) {
            return;
        }
        if (!$this->checkCustomer()) {
            return;
        }
        /*
            // https://www.prestashop.com/forums/topic/7643-how-to-change-default-order-status-for-some-payment-module/
            from config/config.inc.php: list of default status ids
            define('_PS_OS_CHEQUE_', Configuration::get('PS_OS_CHEQUE'));
            define('_PS_OS_PAYMENT_', Configuration::get('PS_OS_PAYMENT'));
            define('_PS_OS_PREPARATION_', Configuration::get('PS_OS_PREPARATION'));
            define('_PS_OS_SHIPPING_', Configuration::get('PS_OS_SHIPPING'));
            define('_PS_OS_DELIVERED_', Configuration::get('PS_OS_DELIVERED'));
            define('_PS_OS_CANCELED_', Configuration::get('PS_OS_CANCELED'));
            define('_PS_OS_REFUND_', Configuration::get('PS_OS_REFUND'));
            define('_PS_OS_ERROR_', Configuration::get('PS_OS_ERROR'));
            define('_PS_OS_OUTOFSTOCK_', Configuration::get('PS_OS_OUTOFSTOCK'));
            define('_PS_OS_OUTOFSTOCK_PAID_', Configuration::get('PS_OS_OUTOFSTOCK_PAID'));
            define('_PS_OS_OUTOFSTOCK_UNPAID_', Configuration::get('PS_OS_OUTOFSTOCK_UNPAID'));
            define('_PS_OS_BANKWIRE_', Configuration::get('PS_OS_BANKWIRE'));
            define('_PS_OS_PAYPAL_', Configuration::get('PS_OS_PAYPAL'));
            define('_PS_OS_WS_PAYMENT_', Configuration::get('PS_OS_WS_PAYMENT'));
            define('_PS_OS_COD_VALIDATION_', Configuration::get('PS_OS_COD_VALIDATION'));
        */

        $state = $params['order']->getCurrentState();
        if (in_array(
            $state,
            array(Configuration::get($this->CONFIG_RIBA_ORDER_STATUS),
                Configuration::get('PS_OS_OUTOFSTOCK'),
                Configuration::get('PS_OS_OUTOFSTOCK_UNPAID')
            ))
        ) {
            $this->smarty->assign(array(
                'total_to_pay' => Tools::displayPrice(
                    $params['order']->getOrdersTotalPaid(),
                    new Currency($params['order']->id_currency),
                    false
                ),
                'shop_name' => $this->context->shop->name,
                'status' => 'ok',
                'id_order' => $params['order']->id,
                'mxb_payment_riba_module_name' => $this->name,
                'payment_name' => $this->ribaName, //Configuration::get($this->CONFIG_RIBA_NAME),
                'payment_instructions' => $this->ribaInstructions, //Configuration::get($this->CONFIG_RIBA_INSTRUCTIONS),
                'payment_days' => $this->ribaDaysCustomer, //Configuration::get($this->CONFIG_RIBA_DAYS),
                'order_state_name' => $this->orderStateName,

            ));
            if (isset($params['order']->reference) && !empty($params['order']->reference)) {
                $this->smarty->assign('reference', $params['order']->reference);
            }
        } else {
            $this->smarty->assign('status', 'failed');
        }
        return $this->fetch('module:' . $this->name . '/views/templates/hook/payment_return.tpl');
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency((int)($cart->id_currency));
        $currencies_module = $this->getCurrency((int)$cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

    public function renderForm()
    {
        $states = OrderState::getOrderStates((int)$this->context->language->id);
        $field_values = $this->getConfigFieldsValues();

        // $customers = Customer::getCustomers();
        
        
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->trans('Contact details', array(), 'Modules.Checkpayment.Admin'),
                    'icon' => 'icon-envelope'
                ),
                'description' => $this->description
                    . '<br><b>' . $this->l('Documentation')
                    . '</b>: <a href="https://www.mxbinformatica.it/prestashop-mxb-delaypay/" target="_blank">'
                    . $this->l('click here') . '</a>',

                'input' => array(
                    array(
                        'type' => 'text',
                        'lang' => true,
                        'label' => $this->l('Payment type (name)'),
                        'desc' => $this->l('Name of the payment (Bankwire, RIBA, check ...). This text will be shown in payment selection after the text "Pay with ".'),
                        'name' => $this->CONFIG_RIBA_NAME,
                        'required' => true
                    ),
                    array(
                        'type' => 'textarea',
                        'lang' => true,
                        'label' => $this->l('Payment instructions'),
                        'desc' => $this->l('Instruction for order confirmation, delayed payment rules, accountholder and address ...'),
                        'name' => $this->CONFIG_RIBA_INSTRUCTIONS,
                        'required' => true
                    ),
                    array(
                        'type' => 'text',
                        'lang' => true,
                        'label' => $this->l('Delayed payment duration in days'),
                        'desc' => $this->l('You can set a quantity in days or a simple text like "as agreed" or other definitions. Text will be added to payment name in checkout area (i.e. Pay with [name][days]).'),
                        'name' => $this->CONFIG_RIBA_DAYS,
                        'required' => true
                    ),

                    array(
                        'type' => 'select',
                        'lang' => true,
                        'label' => $this->l('Order status'),
                        'name' => $this->CONFIG_RIBA_ORDER_STATUS,
                        'desc' => $this->l('Select the desired order status when customer pays with this payment method.'),
                        'options' => array(
                            'query' => $states,
                            'id' => 'id_order_state',
                            'name' => 'name',
                        ),
                    ),
                    // obsoleto
                    /*
                    array(
                        'type' => 'text',
                        'label' => $this->l('Allowed customers list'),
                        'desc' => $this->l('Leave it empty if you want to allow all customers to pay with this module with delay = days set above. To allow to a few customer only to use this module, insert a list of customer ID separated by semicolon or comma. Also you can set a "days" value different for each customer: insert a colon and a value after each customer ID. SAMPLE: "2,4:60,25:as agreed,120". This way customer 2 and 120 will use the default days, customer 4 is 60 days, customer 25 is "as agreed" and no other customers will be able to use the module.'),
                        'name' => $this->CONFIG_RIBA_CUSTOMER_LIST,
                    ),
                    */
                ),
                'submit' => array(
                    'title' => $this->trans('Save', array(), 'Admin.Actions'),
                )
            ),
        );
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->id = (int)Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'btnSubmit';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->table = $this->table;
        $helper->default_form_language = $lang->id;
        $helper->module = $this;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;

        // $helper->override_folder = '/';

        $helper->tpl_vars = array(
            'fields_value' => $field_values,
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
            'base_url' => $this->context->shop->getBaseURL(),
            // 'customers' => $customers,
        );
        $this->fields_form = array();

        return $helper->generateForm(array($fields_form));
    }

    public function renderList()
    {
        $customers = $this->getCustomersFull();

        $this->context->smarty->assign(
            array(
                'link' => $this->context->link,
                'customers' => $customers,
                'name' => $this->customName,
                'table_name' => $this->tableName,
            )
        );

        return $this->display(__FILE__, 'customer_list.tpl');
    }


    public function renderAddForm()
    {
        $idLang = $this->context->language->id;
        // $customers = Customer::getCustomers();
        $customers = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
            '
            SELECT `id_customer`, `email`, `firstname`, `lastname`
            FROM `' . _DB_PREFIX_ . 'customer`
            WHERE 1 ' . Shop::addSqlRestriction(Shop::SHARE_CUSTOMER) . '
            ORDER BY `email` ASC'
        );

        $customer_array = array();
        foreach ($customers as $key=>$customer) {
            $customer_array[$key] = array();
            $customer_array[$key]['id_customer'] = (int)$customer['id_customer'];
            $customer_array[$key]['name'] = $customer['email'] . ' - ' . $customer['firstname'] . ' ' . $customer['lastname'];
        }

        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Customer information'),
                    'icon' => 'icon-cogs',
                ),
                'description' =>
                    '<b>' . $this->displayName . ' ver. ' . $this->version . ' (dev. ' . $this->dev_version . ')</b>'
                    . '<br><ol><li>' . $this->l('Select a customer who have to access to this payment method, and insert related days or comment.') . '</li></ol>',

                'input' => array(
                    array(
                        'type' => 'select',
                        'label' => $this->l('Customer'),
                        'name' => 'id_customer',
                        'options' => array(
                            'query' => $customer_array,
                            'id' => 'id_customer',
                            'name' => 'name',
                        ),
                        'col' => '4',
                    ),

                    array(
                        'type' => 'text',
                        'label' => $this->l('Number of days'),
                        'name' => 'days',
                        'desc' => array(
                            $this->l('You can set a quantity in days or a simple text like "as agreed" or other definitions. Text will be added to payment name in checkout area (i.e. Pay with [name][days]).'),
                        ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->getTranslator()->trans('Save', array(), 'Admin.Actions'),
                )
            ),
        );

        if (Tools::isSubmit('id_delaypay_customer') && $this->getCustomer((int)Tools::getValue('id_delaypay_customer'))) {
            $fields_form['form']['input'][] = array('type' => 'hidden', 'name' => 'id_delaypay_customer');
        }

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();
        $helper->module = $this;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitCustomer';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $language = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->tpl_vars = array(
            'base_url' => $this->context->shop->getBaseURL(),
            'language' => array(
                'id_lang' => $language->id,
                'iso_code' => $language->iso_code
            ),
            'fields_value' => $this->getAddFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        $helper->override_folder = '/';
        /*
        $languages = Language::getLanguages(false);
        if (count($languages) > 1) {
            return $this->getMultiLanguageInfoMsg() . $helper->generateForm(array($fields_form));
        } else {
            return $helper->generateForm(array($fields_form));
        }
        */
        return $helper->generateForm(array($fields_form));
    }


    public function getAddFieldsValues()
    {
        $fields = array();

        if (Tools::isSubmit('id_delaypay_customer') && $this->getCustomer((int)Tools::getValue('id_delaypay_customer'))) {
            $customer = $this->getCustomer((int)Tools::getValue('id_delaypay_customer'));
            $fields['id_delaypay_customer'] = (int)Tools::getValue('id_delaypay_customer', (int)$customer['id_' . $this->tableName]);
        } else {
            $customer = $this->newCustomer();
        }
        $fields['id_customer'] = Tools::getValue('id_customer', $customer['id_customer']);
        $fields['days'] = Tools::getValue('days', $customer['days']);

        return $fields;
    }


    public function getConfigFieldsValues()
    {
        $languages = Language::getLanguages(false);
        $id_lang = (int)Context::getContext()->language->id;
        $fields = array();

        //$fields[$this->CONFIG_RIBA_DAYS] = Tools::getValue($this->CONFIG_RIBA_DAYS, Configuration::get($this->CONFIG_RIBA_DAYS));
        foreach ($languages as $lang) {
            $fields[$this->CONFIG_RIBA_NAME][$lang['id_lang']] = Tools::getValue($this->CONFIG_RIBA_NAME . '_'.$lang['id_lang'], Configuration::get($this->CONFIG_RIBA_NAME, $lang['id_lang']));
            $fields[$this->CONFIG_RIBA_INSTRUCTIONS][$lang['id_lang']] = Tools::getValue($this->CONFIG_RIBA_INSTRUCTIONS . '_'.$lang['id_lang'], Configuration::get($this->CONFIG_RIBA_INSTRUCTIONS, $lang['id_lang']));
            $fields[$this->CONFIG_RIBA_DAYS][$lang['id_lang']] = Tools::getValue($this->CONFIG_RIBA_DAYS . '_'.$lang['id_lang'], Configuration::get($this->CONFIG_RIBA_DAYS, $lang['id_lang']));
        }
        $fields[$this->CONFIG_RIBA_ORDER_STATUS] = Tools::getValue($this->CONFIG_RIBA_ORDER_STATUS, Configuration::get($this->CONFIG_RIBA_ORDER_STATUS));
        // MxbToolsDebug::WriteLog($this->name, 'getConfigFieldsValues', 'before getting customerList', true);

        // customer list string and array
        // $fields[$this->CONFIG_RIBA_CUSTOMER_LIST] = trim(Tools::getValue($this->CONFIG_RIBA_CUSTOMER_LIST, Configuration::get($this->CONFIG_RIBA_CUSTOMER_LIST))); // obsoleto
        $days = $fields[$this->CONFIG_RIBA_DAYS][$id_lang];

        // $customerList = $this->getCustomerListFromString($fields[$this->CONFIG_RIBA_CUSTOMER_LIST], $days);
        /*
        try {
        $customerList = $this->getCustomerListFromString($fields[$this->CONFIG_RIBA_CUSTOMER_LIST], $days);
        }
        catch (Exception $e) {
            MxbToolsDebug::WriteLog($this->name, 'getConfigFieldsValues', 'ERROR getting customerList: ' . $e->getMessage(), true);
        }
        */
        $customerList = $this->getCustomersFull($days);
        $fields[$this->CONFIG_RIBA_CUSTOMER_LIST_ARR] = $customerList;

        return $fields;
    }

    private function getCustomerList($days = null)
    {
        $id_lang = (int)Context::getContext()->language->id;
        if (!$days) {
            $days = Tools::getValue($this->CONFIG_RIBA_DAYS . '_'.$id_lang, Configuration::get($this->CONFIG_RIBA_DAYS, $id_lang));
        }
        $customerList = $this->getCustomers($days);
        if (!$customerList) {
            return null;
        }
        foreach ($customerList as &$customer) {
            $customer['days'] = $this->getDaysString($customer['days'], $days);
        }

        return $customerList;
    }

    /**
     *
     * expected string format: "id_customer:days;id_customer:days ...."
     * expected "single customer" format:
     *                    id_customer
     *                   OR
     *                    id_customer:days or description
     *
     *
     */
     /*
    private function getCustomerListFromString($config_customer_list_string = '', $days = null)
    {
        $id_lang = (int)Context::getContext()->language->id;
        if (!$days) {
            $days = Tools::getValue($this->CONFIG_RIBA_DAYS . '_'.$id_lang, Configuration::get($this->CONFIG_RIBA_DAYS, $id_lang));
        }
        $customerList = array();
        if ( strlen(trim($config_customer_list_string)) > 0 ) {
            $cs = str_replace(",", ";", $config_customer_list_string);
            if (strpos($cs, ";") !== false) {
                $cs_a1 = explode(";", $cs);
                if (count($cs_a1)) {
                    foreach ($cs_a1 as $cust) {
                        $customerList[] = $this->getCustomerInfoFromString($cust, $days);
                    }
                }
            } else {    // string contains one customer only
                $customerList[] = $this->getCustomerInfoFromString($cs, $days);
            }
        }
        // MxbToolsDebug::WriteLog($this->name, 'getCustomerListFromString', 'customerList: ' . print_r($customerList, true), true);

        return $customerList;
    }

    private function getCustomerInfoFromString($customerString = '', $days = 30)
    {
        // expected: id_customer
        //              OR
        //             id_customer:days or description
        $customer = array();
        if (strpos($customerString, ":") !== false) {
            $cust_a1 = explode(":", $customerString);
            $customer = array(
                        'id_customer' => (int)trim($cust_a1[0]),
                        'days' => (isset($cust_a1[1]) ? $cust_a1[1] : $days),
            );
        } else {
            $customer = array(
                'id_customer' => (int)trim($customerString),
                'days' => $days,
            );
        }

        return $customer;
    }
    */
    private function getCustomerInfoFromArray($id_customer = null, $customerArray = null)
    {
        $info = null;
        if (is_array($customerArray) && count($customerArray)) {
            if ($id_customer === null) {
                $id_customer = (int)Context::getContext()->cookie->id_customer;
            }
            // MxbToolsDebug::WriteLog($this->name, 'getCustomerInfoFromArray', 'id_customer ' . $id_customer, true);

            if ($id_customer) {
                foreach ($customerArray as $customer) {
                    if ((int)$customer['id_customer'] == $id_customer) {
                        $info = $customer;
                        break;
                    }
                }
            }
        }
        // MxbToolsDebug::WriteLog($this->name, 'getCustomerInfoFromArray', 'customerInfo: ' . print_r($info, true), true);

        return $info;
    }


    /**
     * check if current customer can access this module
     * return true if no customerArray was set
     * return "days" if current customer is in the array
     * return false if customerArray is set but current customer is not in list
     *
     * return false if current customer is not set (TODO: check also for id_customer=0 to allow enable the module for visitors too)
     */
    private function checkCustomer($id_customer = null, $customerArray = null)
    {
        // get customer array without calling getConfigFieldsValues()
        if ($customerArray === null) {
            $id_lang = (int)Context::getContext()->language->id;
            // $cusList = trim(Tools::getValue($this->CONFIG_RIBA_CUSTOMER_LIST, Configuration::get($this->CONFIG_RIBA_CUSTOMER_LIST)));
            $days = Tools::getValue($this->CONFIG_RIBA_DAYS . '_'.$id_lang, Configuration::get($this->CONFIG_RIBA_DAYS, $id_lang));
            // $customerArray = $this->getCustomerListFromString($cusList, $days);
            $customerArray = $this->getCustomerList($days);
            // $customerArray = $this->getCustomersFull($days);
            // MxbToolsDebug::WriteLog($this->name, 'checkCustomer', 'CUS ARR: ' . print_r($customerArray, true), true);
        }

        if (!is_array($customerArray) || !count($customerArray)) {
            return true;
        }
        if ($id_customer === null) {
            $id_customer = (int)Context::getContext()->cookie->id_customer;
        }
        // MxbToolsDebug::WriteLog($this->name, 'checkCustomer', 'id_customer: ' . $id_customer . ', customerArray: ' . print_r($customerArray, true), true);

        if (!$id_customer) {
            return false;
        }
        foreach ($customerArray as $customer) {
            if ((int)$customer['id_customer'] === (int)$id_customer) {
                return $customer['days'];
            }
        }
        // customer list is filled but curren customer is not in it
        return false;
    }


    public function getPaymentFullName()
    {
        return $this->ribaName . ' ' . $this->ribaDaysCustomer;
    }


    public function getTemplateVars()
    {
        $cart = $this->context->cart;
        $total = $this->trans(
            '%amount% (tax incl.)',
            array(
                '%amount%' => Tools::displayPrice($cart->getOrderTotal(true, Cart::BOTH)),
            ),
            'Modules.Checkpayment.Admin'
        );
        // MxbToolsDebug::WriteLog($this->name, 'getTemplateVars', 'START', true);

        $config = $this->getConfigFieldsValues();
        $id_lang = (int)Context::getContext()->language->id;
        $ribaName = $config[$this->CONFIG_RIBA_NAME][$id_lang];
        if (!$ribaName) {
            $ribaName = '___________';
        }
        $ribaInstructions = Tools::nl2br($config[$this->CONFIG_RIBA_INSTRUCTIONS][$id_lang]);
        if (!$ribaInstructions) {
            $ribaInstructions = '___________';
        }
        // MxbToolsDebug::WriteLog($this->name, 'getTemplateVars', 'before getting customerInfo', true);

        $ribaDays = null;
        $customerInfo = $this->getCustomerInfoFromArray(null, $config[$this->CONFIG_RIBA_CUSTOMER_LIST_ARR]);
        // MxbToolsDebug::WriteLog($this->name, 'getTemplateVars', 'after getting customerInfo: ' . print_r($customerInfo, true), true);

        if ($customerInfo) {
            $ribaDays = $customerInfo['days'];
        } else {
            $ribaDays = Tools::nl2br($config[$this->CONFIG_RIBA_DAYS][$id_lang]);
        }
        if (!$ribaDays) {
            $ribaDays = '___________';
        } else {
            if ((string)(int)$ribaDays === (string)$ribaDays) {
                $ribaDays .= ' ' . $this->l('days');
            }
        }
        // MxbToolsDebug::WriteLog($this->name, 'getTemplateVars', 'ribaDays: ' . $ribaDays, true);

        return [
            'mxb_payment_riba_module_name' => $this->name,
            'checkTotal' => $total,
            'payment_name' => $ribaName,
            'payment_instructions' => $ribaInstructions,
            'payment_days' => $ribaDays,
            'order_state_name' => $this->orderStateName,
        ];
    }
    
    
    protected function checkForUpdates()
    {
        // DEBUG: reset version to create databases:
        // Configuration::updateGlobalValue($this->CONFIG_VERSION, 0);

        // DEBUG: reinstall all module files
        // $this->installModuleFiles();
        // $this->getCustomerList();

        // check for back-office
        //if (!Tools::getIsset('controller'))
        //    return false;
        $version = Configuration::getGlobalValue($this->CONFIG_VERSION);
        if ($version === 0) {
            // module was fresh of installation or upgrade: check if updates are needed
            Configuration::updateGlobalValue($this->CONFIG_VERSION, self::MXB_DEV_VERSION);
            /*
            if (!isset($this->ribaName) || empty($this->ribaName)) {
                // module was fresh of installation: no updates are needed: update current version and exit
                return false;
            }
            */
            return false;
        }
        if ((int)self::MXB_DEV_VERSION <= (int)$version) {
            return false;
        }

        $confirmations = array();
        $errors = array();

        for ($i = $version+1; $i <= self::MXB_DEV_VERSION; $i++) {
            $version = $i;
            switch ($version) {
                case 1:
                    /*
                    // add new database for customers list
                    $this->installModuleFiles();
                    $this->checkMailTemplate();
                    // Creates tables
                    $this->createTables();
                    
                    // import from old system
                    $id_lang = (int)Context::getContext()->language->id;
                    $cusList = trim(Configuration::get($this->CONFIG_RIBA_CUSTOMER_LIST));
                    $days = Configuration::get($this->CONFIG_RIBA_DAYS, $id_lang);
                    $customerArray = $this->getCustomerListFromString($cusList, $days);
                    foreach ($customerArray as $customer) {
                        if (!$this->customerExists((int)$customer['id_customer'])) {
                            $this->addCustomer($customer);
                        }
                    }
                    */
                    break;
                case 2:

                    break;
            }
        }
        // update current version
        Configuration::updateGlobalValue($this->CONFIG_VERSION, $version);
        return true;
    }
}
